#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <assert.h>

/* Funkcija ispisuje poruku i potom informaciju o greski na osnovu errno vrednosti. */
void error_fatal (const char *message){
  perror (message);
  exit (1);
}

/* Program stampa karatkeristike svakog fajla cije je ime navedeno u komandnoj liniji.*/
int main(int argc, char **argv){
  struct stat file_info;
  int j;
  float size;
  char c;

  /* Proverava se da li postoji bar 1 argument komandne linije. */
  assert (argc >= 2);

  /* Prolazi se kroz argumente komandne linije. */
  for (j=1; j<argc; j++){

    /* Resetovanje prava za novi fajl. */
    char file_desc [] = "----------";

    /* Koristi se funkcija lstat a ne stat jer daje informacije o simbolickom linku
       a ne o fajlu na koji pokazuje taj link. */ 
    if (lstat (argv[j], &file_info) < 0){
      /* Kreira se poseban string kako bi se lakse uocilo kod kog fajla je doslo do greske. */
      char error_message [256];
      sprintf (error_message, "lstat error for %s", argv[j]);
      perror (error_message);
      continue;
    }

    /* Proverava se tip fajla pomocu definisanih makroa. */
    if (S_ISREG (file_info.st_mode))
      file_desc [0] = '-';
    else if (S_ISDIR (file_info.st_mode))
      file_desc [0] = 'd';
    else if (S_ISCHR (file_info.st_mode))
      file_desc [0] = 'c';
    else if (S_ISBLK (file_info.st_mode))
      file_desc [0] = 'b';
    else if (S_ISFIFO (file_info.st_mode))
      file_desc [0] = 'f';
    else if (S_ISLNK (file_info.st_mode))
      file_desc [0] = 'l';
    else if (S_ISSOCK (file_info.st_mode))
      file_desc [0] = 's';
    else
      error_fatal ("Unknown type of file");

    /* Proveravaju se prava vlasnika fajla. */
    if (file_info.st_mode & S_IRUSR)
      file_desc [1] = 'r';
    if (file_info.st_mode & S_IWUSR)
      file_desc [2] = 'w';
    if (file_info.st_mode & S_IXUSR)
      file_desc [3] = 'x';

    /* Proveravaju se prava za grupu kojoj pripada fajl. */
    if (file_info.st_mode & S_IRGRP)
      file_desc [4] = 'r';
    if (file_info.st_mode & S_IWGRP)
      file_desc [5] = 'w';
    if (file_info.st_mode & S_IXGRP)
      file_desc [6] = 'x';

    /* Proveravaju se prava za ostale. */
    if (file_info.st_mode & S_IROTH)
      file_desc [7] = 'r';
    if (file_info.st_mode & S_IWOTH)
      file_desc [8] = 'w';
    if (file_info.st_mode & S_IXOTH)
      file_desc [9] = 'x';

    /* Odredjuje se velicina fajla. */
    if (file_info.st_size < 1024){
      size = file_info.st_size;
      c = 'b';
    }
    else if (file_info.st_size > 1024*1024){
      size = file_info.st_size / (1024*1024);
      c = 'M';
    }
    else {
      size = file_info.st_size / 1024;
      c = 'K';
    }

    /* Stampaju se prikupljeni podaci. */
    printf ("%s %5d %7.1f%c %s\n", file_desc, file_info.st_uid, size, c, argv[j]);

  }

  return 0;
}
