#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <assert.h>

/* Funkcija ispisuje poruku i potom informaciju o greski na osnovu errno vrednosti. */
void error_fatal (const char *message){
  perror (message);
  exit (EXIT_FAILURE);
}

int main(int argc, char **argv){
  struct stat file_info;
  if (argc < 3){
    fprintf (stderr, "./postavi_prava file_name octal_constant_for_access_bits\n");
    exit (EXIT_FAILURE);
  }

  int oktalna_prava = atoi(argv[2]);

  /* Izdvajaju se 3 cifre iz prava koja se prenose kao argument.*/
  int stotine = (oktalna_prava / 100) * 8*8;
  int desetice = ((oktalna_prava / 10) % 10) * 8;
  int jedinice = oktalna_prava % 10;

  /* Kreira se odgovarajuca dekadna vrednost za navedena oktalna prava. */
  int dekadna_prava = stotine + desetice + jedinice;

  /* Postavlja se maska na 0 kako bi sva navedena prava bila dodeljena. */
  umask (0);

  /* Pokusava se sa dobijanjem informacija o fajlu. 
     Napomena: ako se provera vrsi pomocu open, moze doci do greske ako 
     nije postavljeno pravo citanja. Bolja je provera sa stat.*/
  if (stat (argv[1], &file_info) < 0){
    
    /* Ako fajl ne postoji kreira se. */
    if (errno == ENOENT){
      if (open (argv[1], O_CREAT | O_EXCL, dekadna_prava) < 0)
	error_fatal ("open file error");
    }
    /* Doslo je do neke druge greske, prekida se program uz odgovarajucu gresku. */
    else {
      char s[30];
      sprintf (s, "open error for file %s\n", argv[1]);
      error_fatal (s);
    }
  }
  /* Fajl je vec postojao, dodeljuju mu se odgovarajuca prava. */
  else {
    if (chmod (argv[1], dekadna_prava) < 0)
      error_fatal ("chmod error");
  }

  return 0;
}
