#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

/* Funkcija ispisuje poruku i potom informaciju o greski na osnovu errno vrednosti. */
void error_fatal (const char *message){
  perror (message);
  exit (1);
}

/* Program demonstrira umask() sistemski poziv. Program kreira dva fajla sa
   istim pravima pristupa, ali sa razlicitim umask vrednostima, pa ce fajlovi
   ipak imati razlicita prava pristupa. Potom se kreira treci fajl i demonstrira
   promena prava pristupa postojecem fajlu. */
int main(){

  /* umask se postavlja na 0 (sva prava su dozvoljena), a fajl se kreira sa pravima pristupa 0644. */
  umask (0); 
  if (creat ("prvi_fajl", S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH) < 0) // umesto drugog argumenta moze da stoji i 0644
    error_fatal ("creat first file");

  /* umask se postavlja na 0077 (zabranjena sva prava za grupu i ostale), a fajl se kreira sa pravima pristupa 600. */
  umask (S_IRWXG | S_IRWXO);
  if (creat ("drugi_fajl", S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH) < 0)
    error_fatal ("creat drugi_fajl");

  /* umask je trenutno 0077, a fajl se kreira sa pravima pristupa 600. */
  if (creat ("treci_fajl", S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH) < 0)
    error_fatal ("creat treci_fajl");

  /* umask je trenutno 0077, ali on ne utice pri menjanju prava za fajl koji vec postoji.
     Zato se prava fajla postavljaju na 0777. */
  if (chmod ("treci_fajl", 0777) < 0)
    error_fatal ("chmod treci_fajl");
  
  return 0;
}
